require 'rest-client'
require 'base64'
require 'nokogiri'

module Hermes
  class BoClient::Client
    class ClientError < StandardError
      attr_reader :response

      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"

        super(message)
      end

      def code
        to_xml&.at_xpath('code')&.text
      end

      def error_message
        to_xml&.at_xpath('message')&.text
      end

      def to_xml
        Nokogiri::XML(response.to_s).at_xpath('//errorResponse')
      end

      def request
        response.request
      end
    end

    class Forbidden < StandardError; end
    class NotFound < StandardError; end

    attr_accessor :base_url
    attr_accessor :user
    attr_accessor :password
    attr_accessor :logger

    def initialize(attrs={})
      @base_url = attrs.fetch(:base_url, BoClient.config.base_url)
      @user     = attrs.fetch(:user, BoClient.config.user)
      @password = attrs.fetch(:password, BoClient.config.password)
      @logger   = attrs.fetch(:logger, BoClient.config.logger)
    end

    def endpoint_for(path)
      base_resource[path]
    end

    def get(path, params={})
      Nokogiri::XML(raw_get(path, params))
    end

    def raw_get(path, params={})
      begin
        endpoint_for(path).get(params)
      rescue RestClient::ExceptionWithResponse => err
        raise ClientError.new(err.response)
      end
    end

    def post(path, payload, params={})
      begin
        e = endpoint_for(path)

        if @logger
          @logger.debug "POST #{e.url}"
          @logger.debug "Payload : #{payload}"
        end
        r = e.post(payload,params)

        @logger&.debug "Response:  #{r}"
        r.to_s
      rescue RestClient::ResourceNotFound => e
        raise NotFound
      rescue RestClient::Forbidden => e
        raise Forbidden
      rescue RestClient::ExceptionWithResponse => e
        if @logger
          @logger.debug "Response:  #{e.response.to_s.force_encoding('UTF-8')}"
        end
        raise ClientError.new(e.response.force_encoding('UTF-8'))
      end
    end

    def base_resource(reload = false)
      if !@base_resource || reload
        auth_header = 'Basic ' + Base64.encode64( "#{@user}:#{@password}" ).chomp
        @base_resource = RestClient::Resource.new(
          @base_url,
          headers: { authorization: auth_header, 'Content-Type' => 'application/xml' },
          log: logger,
          verify_ssl: false
        )
      end
      @base_resource
    end
  end
end
